package edu.umd.dawn.common.exceptions;

import java.io.PrintWriter;
import java.io.StringWriter;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;

@ControllerAdvice
@Component
@Slf4j
public class CustomExceptionHandler {

    @Value("${config.serviceName}")
    private String source;

    private String getStackTrace(Exception exception) {
        StringWriter sw = new StringWriter();
        exception.printStackTrace(new PrintWriter(sw));
        String exceptionAsString = sw.toString();
        return exceptionAsString;
    }

    private void logException(DawnException exception) {
        if (exception.getBase() != null) {
            MDC.put("stackTrace", getStackTrace(exception.getBase()));
            MDC.put("exceptionMessage", exception.getBase().getMessage());
        } else {
            MDC.put("stackTrace", getStackTrace(exception));
            MDC.put("exceptionMessage", exception.getMessage());
        }
        log.error(exception.getMessage());
        MDC.remove("stackTrace");
        MDC.remove("exceptionMessage");
    }

    private ResponseEntity<Object> returnDawnException(DawnException exception) {
        logException(exception);
        return ExceptionResponse.fromDawnException(exception, source).toResponse();
    }

    private ResponseEntity<Object> wrapExceptionAs(RuntimeException exception, DawnExceptionParameters parameters) {
        DawnException wrapped = new DawnException(parameters, exception);
        return returnDawnException(wrapped);
    }

    @ExceptionHandler(BindException.class)
    protected ResponseEntity<Object> handleBindException(BindException ex, WebRequest request) {

        
        FieldError err = ex.getFieldError();
        DawnException wrapped = new DawnException(BaseExceptions.BAD_REQUEST, ex, "Value " + err.getRejectedValue() + " is invalid for field " + err.getField());

        return returnDawnException(wrapped);
    }

    @ExceptionHandler(ArrayIndexOutOfBoundsException.class)
    public ResponseEntity<Object> handleArrayIndexOutOfBoundsException(
            ArrayIndexOutOfBoundsException exception, WebRequest webRequest) {
        return wrapExceptionAs(exception, BaseExceptions.OUT_OF_BOUNDS);
    }

    @ExceptionHandler(RuntimeException.class)
    public ResponseEntity<Object> handleUnHandledException(RuntimeException exception, WebRequest webRequest) {
        return wrapExceptionAs(exception, BaseExceptions.UNHANDLED_INTERNAL_SERVER_ERROR);
    }

    @ExceptionHandler(DawnException.class)
    public ResponseEntity<Object> handleDawnExceptions(DawnException exception, WebRequest webRequest) {
        return returnDawnException(exception);
    }
}
